#!/usr/bin/perl

require 'stat_inc.pl';
require 'g_invest.pl';
require 'common.pl';
use DBI;
use CGI qw/:standard/;
use CGI::Cookie;

$CGI = new CGI;
$pid = getPlayerID();
$charid = $CGI->param("charid");
$charname = getCharName($charid);
$playername = getFullNameFromCharID($charid);
$al = getAccessLevel($pid);



if ($pid != getPIDFromCharID($charid) && $al < $gm_access)
{
   print $CGI->redirect("login.pl?error=" . encodeURL("You are not authorized to validate that character."));
}
else
{
   print $CGI->header;
   ParseForm($header);
   print "<H1>Validating $charname</h1>\n";
   print "<i>($playername)</i><BR><BR>\n";
   
   print "<b>Note:</b>  This page is for review purposes only.  The GMs will be reviewing your STATs and Abilities in much greater detail before the first episode.  Also, the GMs reserve the right to change your STATs and Abilities before the first episode for reasons of plot or play balance.  While we will make every reasonable attempt to inform you of changes and work with you, our decisions in these matters are final.  Our goal is not to screw you over, but to make a better game for everyone involved.<BR><BR>\n";
   
   print "<H2>Validating STATs</H2>\n";
   setSTATvars();
   
   $err_val = 0;
   if ($str < 1 or $str > 6)
   {
      print "<font color= red>Error: Strength of ($str) is out of range.  Must be between 1 and 6, inclusive.</font><BR>\n";
      $err_val++;
   }
   if ($dex < 1 or $dex > 6)
   {
      print "<font color= red>Error: Dexterity of ($dex) is out of range.  Must be between 1 and 6, inclusive.</font><BR>\n";
      $err_val++;
   }
   if ($iq < 1 or $iq > 6)
   {
      print "<font color= red>Error: IQ of ($iq) is out of range.  Must be between 1 and 6, inclusive.</font><BR>\n";
      $err_val++;
   }
   if ($will < 1 or $will > 6)
   {
      print "<font color= red>Error: Will of ($will) is out of range.  Must be between 1 and 6, inclusive.</font><BR>\n";
      $err_val++;
   }
   
   if ($err_val == 0)
   {
      print "Base STATs OK<BR>\n";
   }
   
   $err_val = 0;
   if ($status < 2 or $status > 8)
   {
      print "<font color= red>Error: Status of ($status) is out of range.  Must be between 2 and 8, inclusive.</font><BR>\n";
   }
   else
   {
      print "Status OK.<BR>\n";
   }
   
   if ($holy < 0)
   {
      print "<font color= red>Error: Celestial of ($holy) is out of range.  Must be greater than or equal to 0.</font><BR>\n";
      $err_val++;
   }
   if ($inspiration < 0)
   {
      print "<font color= red>Error: Inspiration of ($inspiration) is out of range.  Must be greater than or equal to 0.</font><BR>\n";
      $err_val++;
   }
   if ($mana < 0)
   {
      print "<font color= red>Error: Mana of ($mana) is out of range.  Must be greater than or equal to 0.</font><BR>\n";
      $err_val++;
   }
   if ($unholy < 0)
   {
      print "<font color= red>Error: Infernal of ($unholy) is out of range.  Must be greater than or equal to 0.</font><BR>\n";
      $err_val++;
   }
   if ($unholy > 0)
   {
      print "<font color=#bbbb00>Warning: You must have GM approval to have an Infernal of $unholy.  This will be manually checked by the GMs.</font><BR>\n";
   }
   if ($unholy > 0 && $holy > 0)
   {
      print "<font color=red>Error: You may not have both Celestial ($holy) and Infernal ($unholy) STATs greater than 0.</font><BR>\n";
      $err_val++;
   }
   if ($mana > 0 && $holy > 0)
   {
      print "<font color=#bbbb00>Warning: Most players may not have both Celestial ($holy) and MANA ($mana) STATs greater than 0.  This must be approved by the GMs</font><BR>\n";
   }

   if ($err_val == 0)
   {
      print "No Errors in Secondary Stats.<BR>\n";
   }
   
   if ($statremain < 0)
   {
      print "<font color=red>Error: You have spent ".($statallowed - $statremain)." points on STATs.  The max you can spend is $statallowed.  This character will not be approved.</font><BR>\n";
   }
   else
   {
      print "STAT points spent OK (Allowed: $statallowed)(".($statallowed - $statremain)." points spent, including Handicaps).<BR>\n";
   }
   
   print "<H2>Validating Abilities</H2>\n";
   $err_val = 0;

   if ($handicaptot > 30)
   {
      print "<font color=#bbbb00>Warning: You have $handicaptot points worth of Handicaps.  More than 20-25 is excessive.  We do not plan to limit Handicaps at this time, but we may if we find that characters are becoming unbalanced.</font><BR>\n";
   }

   local($db) = dbconnect();
   $sql = "select a.*, b.name, b.req_text, b.allowed, b.type, c.basestat from characterabilities a, abilities b, abilitytypes c where a.charid=$charid and a.abid=b.abid and b.type=c.type order by b.type, b.name;";
   #print "$sql<BR>";  
   local ($query) = $db->prepare($sql);
   $query->execute;
   $abcost = 0;
   while ($resptr = $query->fetchrow_hashref())
   {
        $name = $resptr->{"name"};
        $name =~ s/_//g;
	if ($resptr->{"req_text"} == 0 && $resptr->{"text"} ne "")
	{
	  print "<font color=#bbbb00>Warning: $name: Text [".$resptr->{"text"}."] for Ability will probably be ignored.</font><BR>\n";
	}
	if ($resptr->{"req_text"} > 0 && $resptr->{"text"} ne "")
	{
	  print "Note: $name [".$resptr->{"text"}."]: Text for Ability must be approved by the GMs.<BR>\n";
	}
	if ($resptr->{"req_text"} > 0 && $resptr->{"text"} eq "")
	{
	  print "<font color=red>Error: $name: requires text.  No text has been entered.</font><BR>\n";
	  $err_val++;
	}
	
	if ($resptr->{"difficulty"} ne "Cost" && ($resptr->{"level"} < 1 || $resptr->{"level"} > 6))
	{
	  print "<font color=red>Error: $name: Level (".$resptr->{"level"}.") must be between 1 and 6 inclusive.</font><BR>\n";
	  $err_val++;
	}
	
	if ($resptr->{"difficulty"} eq "Varies")
	{
	  print "<font color=red>Error: $name [".$resptr->{"text"}."]: Difficulty is still set to Varies.</font><BR>\n";
	  $err_val++;
	}
	
	local ($chartype) = getCharTypeFromCharID($charid);
	if ($resptr->{"allowed"} eq "None")
	{
	  print "<font color=red>Error: $name: No players are allowed to have this Ability.</font><BR>\n";
	  $err_val++;
	}
	if ($resptr->{"allowed"} eq "NPC Only" && $chartype eq "Player Character")
	{
	  print "<font color=red>Error: $name: Only NPCs are allowed to take this Ability.  </font><BR>\n";
	  $err_val++;
	}
	if ($resptr->{"allowed"} eq "By Request Only" && !abilityallowed($resptr->{"abid"}))
	{ 
	  print "<font color=red>Error: $name: You are not allowed to have this Ability.</font><BR>\n";
	  $err_val++;
	}
	
	$base = $resptr->{"basestat"};
	if (treatasholy($resptr->{"abid"}) > 0)
	{
	    $base = "holy";
	}
	if ($base ne "" && eval("\$".$base) == 0)
	{ 
	  local ($mystr) = $resptr->{"basestat"};
	  $mystr =~ s/unholy/Infernal/g; 
	  $mystr =~ s/holy/Celestial/g; 
	  $mystr =~ s/mana/MANA/g; 
	  if (treatasholy($resptr->{"abid"}) > 0)
	  {
	    $mystr = "Celestial";
	  }
	  print "<font color=red>Error: $name: This Ability requires that you have a $mystr STAT greater than 0.</font><BR>\n";
	  $err_val++;
	}
	
	if ($resptr->{"type"} ne "Handicap")
	{
	  $abcost += $resptr->{"cost"};
	}
   }
   $query->finish;
   
   #combat manuevers
   print "<BR><b>Checking Combat Manuevers.</b><BR>\n";
   $un_cl = getlevel('UNARMED/CLOSE WEAPONS');
   $sql = "select * from characterabilities a, abilities b where a.charid=$charid and a.abid=b.abid and b.type='Combat Manuever' and b.category='Unarmed/Close';";   
   #print "$sql<BR>\n";
   $query = $db->prepare($sql);
   $query->execute;
   $num = $query->rows;
   if ($num > $un_cl)
   {
	  print "<font color=red>Error: You have $num Unarmed/Close Combat Manuevers and you are only allowed $un_cl (your level of the Unarmed/Close Weapons Skill).</font><BR>\n";
	  $err_val++;
   }
   else
   {
	  print "You are allowed $un_cl (your level of the Unarmed/Close Weapons Skill) Unarmed/Close combat manuevers and have taken $num.<BR>\n";
   }
   while ($resptr = $query->fetchrow_hashref())
   {
      if ($resptr->{"level"} > $un_cl)
      {
	  print "<font color=red>Error: Your level of ".$resptr->{"name"}." (".$resptr->{"level"}.") is greater than $un_cl (your level of the Unarmed/Close Weapons Skill).</font><BR>\n";
	  $err_val++;
      }
   }
   $query->finish;
   
   $un_cl = getlevel('RANGED/THROWN WEAPONS');
   $sql = "select * from characterabilities a, abilities b where a.charid=$charid and a.abid=b.abid and b.type='Combat Manuever' and b.category='Ranged';";   
   #print "$sql<BR>\n";
   $query = $db->prepare($sql);
   $query->execute;
   $num = $query->rows;
   if ($num > $un_cl)
   {
	  print "<font color=red>Error: You have $num Ranged Combat Manuevers and you are only allowed $un_cl (your level of the Ranged/Thrown Weapons Skill).</font><BR>\n";
	  $err_val++;
   }
   else
   {
	  print "You are allowed $un_cl (your level of the Ranged/Thrown Weapons Skill) Ranged combat manuevers and have taken $num.<BR>\n";
   }
   while ($resptr = $query->fetchrow_hashref())
   {
      if ($resptr->{"level"} > $un_cl)
      {
	  print "<font color=red>Error: Your level of ".$resptr->{"name"}." (".$resptr->{"level"}.") is greater than $un_cl (your level of the Ranged/Thrown Weapons Skill).</font><BR>\n";
	  $err_val++;
      }
   }
   $query->finish;
   
   $un_cl = getlevel('DODGE');
   $sql = "select * from characterabilities a, abilities b where a.charid=$charid and a.abid=b.abid and b.type='Combat Manuever' and b.category='Defense';";   
   #print "$sql<BR>\n";
   $query = $db->prepare($sql);
   $query->execute;
   $num = $query->rows;
   if ($num > $un_cl)
   {
	  print "<font color=red>Error: You have $num Defense Combat Manuevers and you are only allowed $un_cl (your level of the Dodge Skill).</font><BR>\n";
	  $err_val++;
   }
   else
   {
	  print "You are allowed $un_cl (your level of the Dodge Skill) Defense combat manuevers and have taken $num.<BR>\n";
   }
   $query->finish;
   while ($resptr = $query->fetchrow_hashref())
   {
      if ($resptr->{"level"} > $un_cl)
      {
	  print "<font color=red>Error: Your level of ".$resptr->{"name"}." (".$resptr->{"level"}.") is greater than $un_cl (your level of the Dodge Skill).</font><BR>\n";
	  $err_val++;
      }
   }
   $query->finish;
   
   #weird science
   $ws_err = 0;
   if ($weirdscience > 0)
   {
   	print "<BR><b>Checking Weird Science.</b><BR>\n";
   }
   
   if ($weirdscience > 0 && $inspiration == 0)
   {
   	$ws_err ++;
   	print "<<font color=red>Error:  You have spent ".getcost($weirdscience,2)." points on the Weird Science skill, but have no points of Inspiration.  You must have at least 1 Inspiration to take Weird Science.</font><BR>\n";
   }
   
   if ($weirdscience > 6)
   {
   	$ws_err ++;
   	print "<font color=red>Error:  You are only allowed up to a 6 in Weird Science.  You have taken it at level $weirdscience.</font><BR>\n";
   }
   
   $sql = "select a.*, b.mfname, b.mfsecret from charactermotiveforces a, motiveforces b where a.charid=$charid and a.mfid=b.mfid order by b.mfname;";
   #print "$sql<BR>\n";
   $query = $db->prepare($sql);
   $query->execute;
   $ws_sum = 0;
   while ($resptr = $query->fetchrow_hashref())
   {
     if ($resptr->{"level"} > 6)
     {
   	$ws_err ++;
   	print "<font color=red>Error:  The maximum level you can have in ".$resptr->{"mfname"}." is 6.  You have ".$resptr->{"level"}.".</font><BR>\n";
     }
     if ($resptr->{"mfsecret"} > 0)
     {
   	print "<font color=#bbbb00>Warning: ".$resptr->{"mfname"}." is a secret Motive Force.  You may only have this with GM approval.</font><BR>\n";
     }
     
     $ws_sum += $resptr->{"level"};
   }  
   $query->finish;
   
   $sql = "select a.*, b.aname from characterapplications a, applications b where a.charid=$charid and a.aid=b.aid order by b.aname;";
   #print "$sql<BR>\n";
   $query = $db->prepare($sql);
   $query->execute;
   while ($resptr = $query->fetchrow_hashref())
   {
     if ($resptr->{"level"} > 6)
     {
   	$ws_err ++;
   	print "<font color=red>Error:  The maximum level you can have in ".$resptr->{"aname"}." is 6.  You have ".$resptr->{"level"}.".</font><BR>\n";
     }
     
     $ws_sum += $resptr->{"level"};
   }  
   $query->finish;
   
   if ($ws_sum > $weirdscience * $ws_mult)
   {
   	$ws_err ++;
   	print "<font color=red>Error:  The maximum you can spend on Motive Forces and Applications is ".($weirdscience * $ws_mult)." (your level of Weird Science times $ws_mult).  You have spent $ws_sum points.</font><BR>\n";
   }
   
   if ($ws_err > 0)
   {
   	print "<BR><font color=red>Errors in Weird Science</font><BR>\n";
   }
   elsif ($weirdscience > 0)
   {
   	print "<BR>Weird Science OK</BR>\n";
   }
      
   $err_val += $ws_err;
   
   #abilities cost/spent
   $abcost += getcost($weirdscience,2);
   $abcost += $vlpoints;
   if ($abcost > $aballowed + $statremain)
   {
   	$err_val ++;
   	print "<BR><font color=red>You spent $abcost points on Abilities, Weird Science, and Virtual Locations.  The max you can spend is " . ($aballowed + $statremain) ." ($aballowed + $statremain points from STATs and Handicaps).</font><BR>\n";
   }
   else
   {
   	print "<BR>You are allowed ". ($aballowed + $statremain) ." ($aballowed + $statremain points from STATs and Handicaps) points for Abilities, Weird Science, and Virtual Locations.  You spent $abcost.<BR>\n";
   }

   if ($err_val == 0)
   {
      print "<BR>No Errors in Abilities.<BR>\n";
   }
   else
   {
      print "<BR><font color=red>Errors in Abilities.</font><BR>\n";
   }

   #virtual locations
   print "<H2>Validating Virtual Locations</H2>\n";
   $err_val = 0;

   $sql = "select * from virtuallocations where vlowner=$charid";
   
   $query = $db->prepare($sql);
   $query->execute;
   
   if ($query->rows == 0)
   {
   	print "<font color=#bbbb00>Warning: You own no Virtual Locations.  Where do you sleep?</font><BR>\n";
   }
   
   $vl_tot = 0;
   while ($resptr = $query->fetchrow_hashref())
   {
      $vl_fancy = $resptr->{"vl_fancy"};
      $vl_secure = $resptr->{"vl_secure"};
      $vl_location = $resptr->{"vl_location"};
      $vl_inventory = $resptr->{"vl_inventory"};
      $vl_tot += $vl_fancy + $vl_secure + $vl_location + $vl_inventory; 
      
      $vl_fancy = getVLtotal($resptr->{"vlid"},"vl_fancy","vla_fancy");
      $vl_secure = getVLtotal($resptr->{"vlid"},"vl_secure","vla_secure");
      $vl_location = getVLtotal($resptr->{"vlid"},"vl_location","vla_location");
      $vl_inventory = getVLtotal($resptr->{"vlid"},"vl_inventory","vla_inventory");
      
      if ($vl_fancy < 1)
      {
   	$err_val ++;
   	print "<font color=red>The level of <i>fancy</i> for <b>".$resptr->{"vlname"}."</b> must be at least 1.</font><BR>\n";     
      }
      elsif ($vl_fancy > 14)
      {
   	print "<font color=#bbbb00>Warning: The level of <i>fancy</i> for <b>".$resptr->{"vlname"}."</b> is $vl_fancy.  14 is really the highest it needs to go.  You may keep it at this level if you wish.</font><BR>\n";     
      }
      elsif ($vl_fancy < $status)
      {
   	print "<font color=#bbbb00>Warning: The level of <i>fancy</i> for <b>".$resptr->{"vlname"}."</b> is $vl_fancy.  By making it less than your Status of $status, you are not keeping up apperances and this may hurt you if you host a party or salon there, if you're into that sort of thing.</font><BR>\n";     
      }
      
      if ($vl_secure < 0)
      {
   	$err_val ++;
   	print "<font color=red>The level of <i>secure</i> for <b>".$resptr->{"vlname"}."</b> must be at least 0.</font><BR>\n";     
      }
      elsif ($vl_secure > 14)
      {
   	print "<font color=#bbbb00>Warning: The level of <i>secure</i> for <b>".$resptr->{"vlname"}."</b> is $vl_secure.  14 is really the highest it needs to go.  You may keep it at this level if you wish.</font><BR>\n";     
      }
      elsif ($vl_secure < 5)
      {
   	print "<font color=#bbbb00>Warning: The level of <i>secure</i> for <b>".$resptr->{"vlname"}."</b> is $vl_secure.  By making it less than 5, you are making it fairly easy for people with the \"Search Virtual Location\" skill to find things there.</font><BR>\n";     
      }
      
      if ($vl_location < 1)
      {
   	$err_val ++;
   	print "<font color=red>The level of <i>location</i> for <b>".$resptr->{"vlname"}."</b> must be at least 1.</font><BR>\n";     
      }
      elsif ($vl_location > 14)
      {
   	print "<font color=#bbbb00>Warning: The level of <i>location</i> for <b>".$resptr->{"vlname"}."</b> is $vl_location.  14 is really the highest it needs to go.  You may keep it at this level if you wish.</font><BR>\n";     
      }
      elsif ($vl_location < $status)
      {
   	print "<font color=#bbbb00>Warning: The level of <i>location</i> for <b>".$resptr->{"vlname"}."</b> is $vl_location.  By making it less than your Status of $status, you are saying that it is in a poorer or less fashionable area and this may hurt you if you host a party or salon there, if you're into that sort of thing.</font><BR>\n";     
      }
      
      if ($vl_inventory < 0)
      {
   	$err_val ++;
   	print "<font color=red>The level of <i>inventory</i> for <b>".$resptr->{"vlname"}."</b> must be at least 0.</font><BR>\n";     
      }
      elsif ($vl_inventory > 14)
      {
   	print "<font color=#bbbb00>Warning: The level of <i>inventory</i> for <b>".$resptr->{"vlname"}."</b> is $vl_inventory.  14 is really the highest it needs to go.  You may keep it at this level if you wish.</font><BR>\n";     
      }
      elsif ($vl_inventory < 5)
      {
   	print "<font color=#bbbb00>Warning: The level of <i>inventory</i> for <b>".$resptr->{"vlname"}."</b> is $vl_inventory.  By making it less than 5, you are unlikely to be able to get items from this location.</font><BR>\n";     
      }

   }  
   $query->finish;
   
   $sql = "select sum(a.vla_fancy) + sum(a.vla_secure) + sum(a.vla_location) + sum(a.vla_inventory) from VLaccess a where a.charid=$charid;";
   
   #print "$sql<BR>";
   
   $query = $db->prepare($sql);
   $query->execute;
   
   @array = $query->fetchrow_array;
   $vl_tot += $array[0];
   
   $query->finish;
   
   $sql = "select sum(b.vlslevel) from VLstatlink b where b.charid=$charid;";
   
   #print "$sql<BR>";
   
   $query = $db->prepare($sql);
   $query->execute;
   
   @array = $query->fetchrow_array;
   $vl_tot += $array[0];
   
   $query->finish;
   
   $vl_mult = $status;
   
   if ($vl_tot > ($status + $vlpoints) * $vl_mult)
   {
   	$err_val ++;
   	print "<font color=red>You spent $vl_tot Virtual Location Points.  The max you can spend is " . (($status + $vlpoints) * $vl_mult) ." (Your Status of $status plus the $vlpoints you spent all multiplied by $vl_mult).</font><BR>\n";     
   }
   
   #warded
   $sql = "select a.vlslevel, b.vlname from VLstatlink a,virtuallocations b where a.vlid=b.vlid and a.charid=$charid and a.vlsid=2 and a.vlslevel != 0;";
   $query = $db->prepare($sql);
   $query->execute;
   if ($query->rows > 0)
   {
   	@array = $query->fetchrow_array;
   	$err_val ++;
   	print "<font color=red>You may not put points into <i>warded</i> for <b>$array[1]</b> unless you know the WARD spell. Sorry.</font><BR>\n";     
   }
   $query->finish;
   
   #consecrate
   $sql = "select a.vlslevel, b.vlname from VLstatlink a,virtuallocations b where a.vlid=b.vlid and a.charid=$charid and a.vlsid=3 and a.vlslevel != 0;";
   $query = $db->prepare($sql);
   $query->execute;
   if ($query->rows > 0)
   {
   	@array = $query->fetchrow_array;
   	$err_val ++;
   	print "<font color=red>You may not put points into <i>consecrated</i> for <b>$array[01]</b> unless you know the CONSECRATE Celestial Ability. Sorry.</font><BR>\n";     
   }
   $query->finish;
   
   #power spot
   $sql = "select a.vlslevel, b.vlname from VLstatlink a,virtuallocations b where a.vlid=b.vlid and a.charid=$charid and a.vlsid=4 and a.vlslevel != 0;";
   $query = $db->prepare($sql);
   $query->execute;
   if ($query->rows > 0)
   {
   	@array = $query->fetchrow_array;
   	$err_val ++;
   	print "<font color=red>You may not put points into <i>power spot</i> for <b>$array[1]</b> unless you know the EMPOWER LOCATION spell. Sorry.</font><BR>\n";     
   }
   $query->finish;
   
   #unhallowed
   $sql = "select a.vlslevel, b.vlname from VLstatlink a,virtuallocations b where a.vlid=b.vlid and a.charid=$charid and a.vlsid=7 and a.vlslevel != 0;";
   $query = $db->prepare($sql);
   $query->execute;
   if ($query->rows > 0)
   {
   	@array = $query->fetchrow_array;
   	$err_val ++;
   	print "<font color=red>You may not put points into <i>unhallowed</i> for <b>$array[1]</b> unless you know the TAINT LOCATION Infernal Ability. Sorry.</font><BR>\n";     
   }
   $query->finish;
   
   if ($err_val == 0)
   {
      print "<BR>No Errors in Virtual Locations.<BR>\n";
   }
   else
   {
      print "<BR><font color=red>Errors in Virtual Locations.</font><BR>\n";
   }

   #resources/economics
   print "<H2>Validating Occupation</H2>\n";
   
   $numoccs = dosql("select count(oid) from characteroccupations where charid=$charid;") + 0;
   
   if ($numoccs > 1)
   {
   	print "<font color=#bbbb00>Warning: You have multiple occupations.  This is legal, but you will lose BGA slots.</font><BR><BR>\n";     
   }
   if ($numoccs == 0)
   {
   	print "<font color=#bbbb00>Warning: You have no occupation.  This is legal, but unless your character is very upper or very lower class, you may wish to reconsider.</font><BR><BR>\n";     
   }
   else
   {
   	$err_val = 0;
        $sql = "select * from occupations a join characteroccupations b on a.oid=b.oid where charid=$charid order by a.oname;";
        local($query) = $db->prepare($sql);
        $query->execute;
        while ($resptr = $query->fetchrow_hashref())
        {
           $oid = $resptr->{"oid"};
           $oname = $resptr->{"oname"};
           print "<H3>Validating $oname</H3>";
	   $my_occ_errs = validateoccupation($charid,$oid);
	   if ($my_occ_errs >= 0)
	   {
	      print "No Errors in $oname.<BR><BR>\n";
	      $occ_errs[$oid] = $my_occ_errs;
	   }
	   else
	   {
	      print "<font color=red>Errors in $oname.</font><BR><BR>\n";
	      print "$occstr<BR><BR>\n";
	      $occ_errs[$oid] = $my_occ_errs;
	      $err_val -= $my_occ_errs;
	   }
        }
        $query->finish();
	if ($err_val == 0)
	{
	      print "<BR>No Errors in Occupations.<BR>\n";
	}
	else
	{
	      print "<BR><font color=red>Errors in Occupations.</font><BR>\n";
	}
   }
      
   $pounds = 0;
   
   print "<H2>Validating Economy</H2>\n";
   print "<i>What follows is your results from a sample BGP</i><BR><BR>\n";
   print "You start with $pounds.<BR><BR>\n";   
   print "<b>Step One/Three: Occupation/Occupation BGAs</b><BR><BR>\n";
   
   $tot = 0;
   $sql = "select * from occupations a join characteroccupations b on a.oid=b.oid where charid=$charid order by a.oname;";
   local($query) = $db->prepare($sql);
   $query->execute;
   while ($resptr = $query->fetchrow_hashref())
   {
      $job_tot = 0;
      $oid = $resptr->{"oid"};
      $oname = $resptr->{"oname"};
      if ($occ_errs[$oid] < 0) 
      {
   	print "Because of errors in $oname, you receive no income from it this BGP<BR><BR>\n";
      }
      else
      {
        print "$oname:<BR><BR>\n";
        $base_income = $resptr->{"base_income"};
        $bd10_base_income = $resptr->{"d10_base_income"};
      	$num = $base_income + d11($d10_base_income);
     	$job_tot += $num;
    	print $charname." will receive a base income of between $base_income and ".($base_income  + ($d10_base_income * 10))." each BGP.  This BGP, the income is $num.<BR><BR>\n";
    	
    	$effstatus = $status + geteffectivestatus($charid);
    
        $req_status = $resptr->{"req_status"}; 
        $base_stigma_modifier = $resptr->{"base_stigma_modifier"}; 
        $d10_stigma_modifier = $resptr->{"d10_stigma_modifier"}; 
	if ($effstatus < $req_status)
	{
		$amt = $req_status - $effstatus;
		$num = ($base_stigma_modifier * $amt) + d11($d10_stigma_modifier * amt);
		$job_tot -= $num;

		print "Because $charname has an Effective Status of $effstatus, which is less than $req_status, $charname lost $num this BGP.<BR><BR>\n";
	}
	
        $base_bga_income = $resptr->{"base_bga_income"};
        $d10_bga_income = $resptr->{"d10_bga_income"};
	$amt = int(rand(3)) + 1;
	$num = ($base_bga_income * $amt) + d11($d10_bga_income * $amt);
	$job_tot += $num;
	print $charname." will receive an income of between $base_bga_income and ".($base_bga_income  + ($d10_bga_income * 10))." for each BGA.  Assume $amt BGAs performed as a $oname. The income from BGAs is $num.<BR><BR>\n";

        $base_skill_bonus = $resptr->{"base_skill_bonus"};
        $d10_skill_bonus = $resptr->{"d10_skill_bonus"};
	$ab_diff = $occ_errs[$oid];
	if ($ab_diff > 0)
	{
	  $num = ($base_skill_bonus * $ab_diff) + d11($d10_skill_bonus * $ab_diff);
	  $job_tot += $num;

	  print "$charname is very skilled as a $oname and receives a bonus $num this BGP.<BR><BR>\n";
	}


	print "Total money earned this BGP in $oname is $job_tot.<BR><BR>\n";
	$pounds += $job_tot;
   	print "You now have $pounds.<BR><BR>\n";   
      }
   }
   $query->finish;
   
   print "<b>Step Two: Investments</b><BR><BR>\n";
   $sql = "select b.name, a.text, a.level, c.level from characterabilities a, abilities b, characterresourcetypelink c where a.charid=$charid and a.abid=b.abid and b.type='Resource' and c.charid=$charid and c.abid=a.caid and c.typeid=5 order by b.name;"; #5 is income
   #print "$sql<BR>";
   $query = $db->prepare($sql);
   $query->execute;
   $tot = 0;
   while (@array = $query->fetchrow_array())
   {
        $name = $array[0];
        $name =~ s/_//g;
        $text = $array[1];
        $mylev = $array[2];
        $reslev = $array[3];
        handleinvestment();       
   }
   $query->finish;
   
   $sql = "select b.name, d.text, d.level, c.level from keyrolecharacters a, abilities b, keyrolerresourcetypelink c, keyroleabilities d where a.charid=$charid and a.krid=d.krid and d.abid=b.abid and b.type='Resource' and c.krid=a.krid and c.abid=d.kraid and c.typeid=5 order by b.name;"; #5 is income
   #print "$sql<BR>";
   $query = $db->prepare($sql);
   $query->execute;
   while (@array = $query->fetchrow_array())
   {
        $name = $array[0];
        $name =~ s/_//g;
        $text = $array[1];
        $mylev = $array[2];
        $reslev = $array[3];
        handleinvestment();       
   }
   $query->finish;

   print "<BR>You made $tot in Investments.<BR>\n";
   $pounds += $tot;
   print "<BR>You now have $pounds.<BR><BR>\n";   
   
   print "<b>Step Three: BGAs</b><BR><BR>\n";

   print "Handled in Step One: Occupation.<BR><BR>\n";

   print "<b>Step Four: Taxes</b><BR><BR>\n";

   if (hasability("TAX EXEMPT") > 0)
   {
   	print "You are exempt from Taxes.<BR><BR>\n";   
   }
   else
   {
	   $tot = int($pounds * 0.05);
	   if ($tot < 0)
	   {
	     $tot = 0;
	   }
	   print "You paid $tot in Taxes.<BR><BR>\n";
	   $pounds -= $tot;
   }
   print "You now have $pounds.<BR><BR>\n";   
   
   print "<b>Step Five: Expenses</b><BR><BR>\n";

   $num = getlevel("EXPENSES");
   if ($num == 0)
   {
   	print "You have no expenses.<BR><BR>\n";   
   }
   else
   {
	   $tot = $num * $status * 10;
	   print "You owe $tot in Expenses.<BR><BR>\n";
	   $pounds -= $tot;
   }
   print "You now have $pounds.<BR><BR>\n";   
   
   print "<b>Step Six: Virtual Locations</b><BR><BR>\n";

   $tot = $vl_tot * $status;
   print "You put $vl_tot points into Virtual Locations, costing you $tot<BR><BR>\n";
   $pounds -= $tot;
   print "<i>You end up with $pounds.</i><BR><BR>\n";   
   

   print "<BR><a href=\"javascript:history.go(-1)\">Return</a>.<BR>\n";

   print "<br><hr><a href=\"login.pl\">Main menu</a>."; 
   ParseForm($footer);
   $db->disconnect;
}

exit(0);


sub treatasholy()
{
	local ($abid) = $_[0];
	local ($sql) = "select charid from charabilityallowed where abid=$abid and charid=$charid and asholy=1;";
	#print "$sql<BR>";
	local($query) = $db->prepare($sql);
	$query->execute;
	$retval = -1;
	if ($query->rows > 0)
	{
	  local (@array);
	  while (@array = $query->fetchrow_array())
	  {
	    if ("$array[0]" eq "$charid")
	    {
	       $retval = $pid;
	    }
	  }
	}
	$query->finish;
	return $retval;
}

sub handleinvestment()
{
    	$amt = int(rand(10)) + int(rand(10)) + 2;
    	$showres = 1;
    	$newpo = 0;
    	$mult = int($mylev * $mylev * ($status / 2) * ($status /2));
    	
    	if ($name eq "AGRICULTURAL INVESTMENTS: ")
    	{
	  $lowered = 0;
    	  do 
    	  {
    	     $amt = int(rand(10)) + int(rand(10)) + 2;
	     $amt = $inv_ag[$amt];
	     $amt = $res[$reslev][$amt];
	     if ($amt < $eliminate)
	     {
	        $newpo = $amt * $mult;
	     }
	     elsif ($amt == $eliminate)
	     {
	        print "$name $text produced so poorly that it was eliminated<BR>\n";
	        $showres = 0;
	     }
	     elsif ($amt == $raiselev && $levels[$reslev] eq "Average" )
	     {
		$amt = 3;
	        $newpo = $amt * $mult;
	     }
	     elsif ($amt == $lowerlev && ($levels[$reslev] eq "Low" || $lowered > 0))
	     {
		$amt = -2;
	        $newpo = $amt * $mult;
	     }
	     elsif ($amt == $lowerlev)
	     {
	        print "Lowering Profitability of $name $text from $levels[$reslev]";
	        $reslev = $reslev - 1;
	        if ($reslev == 0)
	        {
	           $reslev = 5;
	        }
	        print " to $levels[$reslev]<BR>\n";
	        $lowered= 1;
	     }
	     elsif ($amt == $raiselev)
	     {
	        print "Raising Profitability of $name $text from $levels[$reslev]";
	        $reslev = $reslev + 1;
	        if ($reslev == 6)
	        {
	           $reslev = 1;
	        }
	        print " to $levels[$reslev]<BR>\n";
	     }
	  } while ($amt == $lowerlev || $amt == $raiselev);
    	}
    	
    	if ($name eq "INDUSTRIAL INVESTMENTS: ")
    	{
	  $lowered = 0;
    	  do 
    	  {
    	     $amt = int(rand(10)) + int(rand(10)) + 2;
    	     $amt += getlevel("BUSINESS");
    	     if ($amt > 20)
    	     {
    	       $amt = 20;
    	     }
	     $amt = $inv_in[$amt];
	     $amt = $res[$reslev][$amt];
	     if ($amt < $eliminate)
	     {
	        $newpo = $amt * $mult;
	     }
	     elsif ($amt == $eliminate)
	     {
	        print "$name $text produced so poorly that it was eliminated<BR>\n";
	        $showres = 0;
	     }
	     elsif ($amt == $raiselev && $levels[$reslev] eq "High" )
	     {
		$amt = 3;
	        $newpo = $amt * $mult;
	     }
	     elsif ($amt == $lowerlev && $lowered > 0)
	     {
		#reroll
	        $lowered += 1;
	     }
	     elsif ($amt == $lowerlev)
	     {
	        print "Lowering Profitability of $name $text from $levels[$reslev]";
	        $reslev = $reslev - 1;
	        if ($reslev == 0)
	        {
	           $reslev = 5;
	        }
	        print " to $levels[$reslev]<BR>\n";
	        $lowered = 10;
	     }
	     elsif ($amt == $raiselev)
	     {
	        print "Raising Profitability of $name $text from $levels[$reslev]";
	        $reslev = $reslev + 1;
	        if ($reslev == 6)
	        {
	           $reslev = 1;
	        }
	        print " to $levels[$reslev]<BR>\n";
	     }
	  } while ($amt == $lowerlev || $amt == $raiselev);
    	}
    	
    	if ($name eq "TRADE INVESTMENTS: ")
    	{
	  $lowered = 0;
    	  do 
    	  {
    	     $amt = int(rand(10)) + int(rand(10)) + 2;
    	     $amt += getlevel("HAGGLE");
    	     if ($amt > 20)
    	     {
    	       $amt = 20;
    	     }
	     $amt = $inv_tr[$amt];
	     $amt = $res[$reslev][$amt];
	     if ($amt < $eliminate)
	     {
	        $newpo = $amt * $mult;
	     }
	     elsif ($amt == $eliminate)
	     {
	        print "$name $text produced so poorly that it was eliminated<BR>\n";
	        $showres = 0;
	     }
	     elsif ($amt == $lowerlev && $lowerlev > 0)
	     {
	        $lowered++;
	     }
	     elsif ($amt == $lowerlev)
	     {
	        print "Lowering Profitability of $name $text from $levels[$reslev]";
	        $reslev = $reslev - 1;
	        if ($reslev == 0)
	        {
	           $reslev = 5;
	        }
	        print " to $levels[$reslev]<BR>\n";
	        $lowered++;
	     }
	     elsif ($amt == $raiselev)
	     {
	        print "Raising Profitability of $name $text from $levels[$reslev]";
	        $reslev = $reslev + 1;
	        if ($reslev == 6)
	        {
	           $reslev = 1;
	        }
	        print " to $levels[$reslev]<BR>\n";
	     }
	  } while ($amt == $lowerlev || $amt == $raiselev);
	}
        
        if ($showres == 1)
        {
           print "$name $text at level $mylev, $levels[$reslev] produces $newpo<BR>\n";
           $tot += $newpo;
        }
}